@@ -130,42 +130,6 @@ add_entry (GtkWidget *box, GtkSizeGroup *group,
   return entry;
 }
 
-static void
-pill_response (GtkDialog *dialog, gint response, gpointer unused)
-{
-  pop_dialog (GTK_WIDGET (dialog));
-  gtk_widget_destroy (GTK_WIDGET (dialog));
-
-  if (red_pill_mode != (response == GTK_RESPONSE_YES))
-    {
-      red_pill_mode = (response == GTK_RESPONSE_YES);
-      save_settings ();
-
-      set_settings_menu_visible (red_pill_mode);
-      set_install_from_file_menu_visible (red_pill_mode);
-      update_backend_options ();
-      if (red_pill_show_all || red_pill_show_magic_sys)
-        get_package_list ();
-    }
-}
-
-static void
-ask_the_pill_question ()
-{
-  GtkWidget *dialog;
-
-  dialog =
-    hildon_note_new_confirmation_add_buttons (NULL, 
-					      "Which pill?",
-					      "Red", GTK_RESPONSE_YES,
-					      "Blue", GTK_RESPONSE_NO,
-					      NULL);
-  push_dialog (dialog);
-  g_signal_connect (dialog, "response",
-		    G_CALLBACK (pill_response), NULL);
-  gtk_widget_show_all (dialog);
-}
-
 struct get_catalogues_closure {
   void (*cont) (xexp *catalogues, void *data);
   void *data;
@@ -341,9 +305,12 @@ struct cat_dialog_closure {
   xexp *catalogues_xexp;
   bool dirty;
   bool show_only_errors;
+  bool has_failing_catalogues;
 
   bool showing_catalogues;
 
+  GtkWidget *dialog;
+
   catcache *selected_cat;
   GtkTreeIter selected_iter;
 
@@ -413,15 +380,14 @@ remove_cat_cont (bool res, gpointer data)
 static void
 cat_edit_response (GtkDialog *dialog, gint response, gpointer clos)
 {
-  bool should_ask_the_pill_question = false;
-
   cat_edit_closure *c = (cat_edit_closure *)clos;
+  cat_dialog_closure *d = c->cat_dialog;
 
   if (c->type == cat_readonly) // it cames from an .install file
     ;
   else if (response == REPO_RESPONSE_REMOVE)
     {
-      catcache *cat = c->cat_dialog->selected_cat;
+      catcache *cat = d->selected_cat;
       if (cat == NULL)
 	return;
 
@@ -430,7 +396,7 @@ cat_edit_response (GtkDialog *dialog, gint response, gpointer clos)
 
       remove_cat_clos *rc = new remove_cat_clos;
       rc->loop = loop;
-      rc->cat_dialog = c->cat_dialog;
+      rc->cat_dialog = d;
       rc->catalogue = cat->catalogue_xexp;
 
       ask_yes_no (text, remove_cat_cont, rc);
@@ -446,8 +412,8 @@ cat_edit_response (GtkDialog *dialog, gint response, gpointer clos)
         hildon_check_button_get_active (HILDON_CHECK_BUTTON
                                         (c->disabled_button));
       xexp_aset_bool (c->catalogue, "disabled", disabled);
-      set_cat_list (c->cat_dialog, &c->cat_dialog->selected_iter);
-      c->cat_dialog->dirty = true;
+      set_cat_list (d, &d->selected_iter);
+      d->dirty = true;
     }
   else if (response == GTK_RESPONSE_OK && c->type == cat_editable)
     {
@@ -508,16 +474,16 @@ cat_edit_response (GtkDialog *dialog, gint response, gpointer clos)
       if (all_whitespace (dist))
         dist = NULL;
 
-      reset_cat_list (c->cat_dialog);
+      reset_cat_list (d);
       if (c->isnew)
-        xexp_append_1 (c->cat_dialog->catalogues_xexp, c->catalogue);
+        xexp_append_1 (d->catalogues_xexp, c->catalogue);
       set_catalogue_name (c->catalogue, name);
       xexp_aset_bool (c->catalogue, "disabled", disabled);
       xexp_aset_text (c->catalogue, "components", comps);
       xexp_aset_text (c->catalogue, "dist", dist);
       xexp_aset_text (c->catalogue, "uri", uri);
-      set_cat_list (c->cat_dialog, &c->cat_dialog->selected_iter);
-      c->cat_dialog->dirty = true;
+      set_cat_list (d, &d->selected_iter);
+      d->dirty = true;
 
       g_free (uri);
       g_free (dist);
@@ -526,8 +492,6 @@ cat_edit_response (GtkDialog *dialog, gint response, gpointer clos)
   else if (c->isnew)
     {
       xexp_free (c->catalogue);
-      if (!strcmp (gtk_entry_get_text (GTK_ENTRY (c->uri_entry)), "matrix"))
-	should_ask_the_pill_question = true;
     }
 
   delete c;
@@ -535,8 +499,9 @@ cat_edit_response (GtkDialog *dialog, gint response, gpointer clos)
   pop_dialog (GTK_WIDGET (dialog));
   gtk_widget_destroy (GTK_WIDGET (dialog));
 
-  if (should_ask_the_pill_question)
-    ask_the_pill_question ();
+  /* Emit response signal if needed */
+  if (d && d->dialog && d->show_only_errors && !d->has_failing_catalogues)
+    gtk_dialog_response (GTK_DIALOG (d->dialog), GTK_RESPONSE_CLOSE);
 }
 
 struct scroll_to_params {
@@ -649,8 +614,8 @@ show_cat_edit_dialog (cat_dialog_closure *cat_dialog, xexp *catalogue,
   if (detail)
     gtk_widget_set_size_request (scrolledw, -1, 350);
   else
-    g_object_set (G_OBJECT(scrolledw),
-                  "size-request-policy", HILDON_SIZE_REQUEST_CHILDREN, NULL);
+    hildon_pannable_area_set_size_request_children
+      (HILDON_PANNABLE_AREA (scrolledw));
 
   vbox = gtk_vbox_new (FALSE, 0);
   hildon_pannable_area_add_with_viewport (HILDON_PANNABLE_AREA (scrolledw),
@@ -803,7 +768,7 @@ cat_icon_func (GtkTreeViewColumn *column,
     }
 
   /* Select icon to show when showing the application catalogue dialog */
-  if (!cd->show_only_errors && c->refresh_failed)
+  if (!cd->show_only_errors && c && c->refresh_failed)
     browser_pixbuf = fail_browser_pixbuf;
   else
     browser_pixbuf = ok_browser_pixbuf;
@@ -827,6 +792,9 @@ cat_text_func (GtkTreeViewColumn *column,
 
   gtk_tree_model_get (model, iter, 0, &c, -1);
 
+  if (c == NULL)
+    return;
+
   /* set 'failed catalogue' suffix when needed */
   if (!cd->show_only_errors && c->refresh_failed)
     {
@@ -1022,6 +990,7 @@ static void
 set_cat_list (cat_dialog_closure *c, GtkTreeIter *iter_to_select)
 {
   gint position = 0;
+  guint n_failed_cats = 0;
   catcache **catptr = &c->caches;
   GtkTreePath *path_to_select = NULL;
 
@@ -1056,7 +1025,12 @@ set_cat_list (cat_dialog_closure *c, GtkTreeIter *iter_to_select)
   for (xexp *catx = xexp_first (c->catalogues_xexp); catx;
        catx = xexp_rest (catx))
     {
-      if (c->show_only_errors && !cat_has_errors (catx))
+      gboolean has_errors = cat_has_errors (catx);
+
+      if (has_errors)
+        n_failed_cats++;
+
+      if (c->show_only_errors && !has_errors)
 	continue;
 
       catcache *cat = make_catcache_from_xexp (c, catx);
@@ -1098,6 +1072,9 @@ set_cat_list (cat_dialog_closure *c, GtkTreeIter *iter_to_select)
 	}
     }
 
+  /* Set flag to know there are failing catalogues */
+  c->has_failing_catalogues = (n_failed_cats > 0);
+
   /* Set the focus in the right list element */
   GtkTreeSelection *tree_selection =
     gtk_tree_view_get_selection (GTK_TREE_VIEW (c->tree));
@@ -1196,6 +1173,8 @@ cat_response (GtkDialog *dialog, gint response, gpointer clos)
       current_cat_dialog_clos = NULL;
       hide_updating ();
 
+      g_object_unref (c->dialog);
+
       delete c;
     }
 }
@@ -1268,6 +1247,7 @@ show_catalogue_dialog (xexp *catalogues,
   c->catalogues_xexp = catalogues;
   c->dirty = false;
   c->show_only_errors = show_only_errors;
+  c->has_failing_catalogues = false;
   c->showing_catalogues = false;
   c->selected_cat = NULL;
   c->cont = cont;
@@ -1277,7 +1257,10 @@ show_catalogue_dialog (xexp *catalogues,
 
   current_cat_dialog_clos = c;
 
+  /* Create dialog (and save reference) */
   dialog = gtk_dialog_new ();
+  c->dialog = dialog;
+  g_object_ref (dialog);
 
   if (show_only_errors)
     gtk_window_set_title (GTK_WINDOW (dialog), _("ai_ti_failed_repositories"));
@@ -1361,7 +1344,7 @@ add_catalogues_cont_3 (bool res, bool enable, void *data)
 	  g_assert (c->cur);
 	  xexp_aset_bool (c->cur, "disabled", false);
 	}
-      else 
+      else
 	{
 	  if (c->cur)
 	    xexp_del (c->catalogues, c->cur);
@@ -1436,16 +1419,25 @@ add_catalogues_cont_2 (add_catalogues_closure *c)
 
       c->cur = find_catalogue (c->catalogues, c->rest);
 
-      if (!c->update || c->cur == NULL)
-	{
-	  // New version should be added
-	  cont = add_catalogues_cont_3_add;
-	}
-      else if (c->cur && xexp_aref_bool (c->cur, "disabled"))
+      if (c->cur && xexp_aref_bool (c->cur, "disabled"))
 	{
 	  // Old version should be enabled
 	  cont = add_catalogues_cont_3_enable;
 	}
+      else if (!c->update || c->cur == NULL)
+	{
+          // Let's remove useless data
+          xexp_adel (c->rest, "file");
+          xexp_adel (c->rest, "id");
+
+          if (xexp_aref_text (c->rest, "uri") != NULL)
+            {
+              // New version should be added
+              cont = add_catalogues_cont_3_add;
+            }
+          else
+            cont = NULL;
+	}
       else
 	cont = NULL;
 
